/*
 * Cluster Information Service - a monitoring system for Linux clusters
 * Copyright (C) 2000 Institute of Informatics, Slovak Academy of Sciences.
 * Written by Jan Astalos (astalos.ui@savba.sk)
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as published
 * by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Kernel module for monitoring of system information.
 */

#include <linux/config.h>
#include <linux/module.h>

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif

#include <asm/uaccess.h>
#include <net/sock.h>
#include <linux/swap.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <linux/version.h>
#include <linux/kernel_stat.h>

#include "cis.h"

#define LOAD_CONV(a) ((a+(FIXED_1/200)) / (FIXED_1/100))

static int sysinfo_read_proc(char *page, char **start, off_t offset, int count, int *eof, void *data)
{
        struct sysinfo si;
        struct cis_hostinfo di;
	int major, disk;
	int len = 0;
	
        memset((char *)&di, 0, HOSTINFO_LEN);

	di.idle = init_tasks[0]->times.tms_utime + init_tasks[0]->times.tms_stime;

	for (major = 0; major < DK_MAX_MAJOR; major++)
		for (disk = 0; disk < DK_MAX_DISK; disk++) {
			di.disks_read  += kstat.dk_drive_rblk[major][disk] >> 1;
                        di.disks_write += kstat.dk_drive_wblk[major][disk] >> 1;
		}

	di.ctx_swtch = kstat.context_swtch;
	di.swpin  = kstat.pswpin << (PAGE_SHIFT - 10);
	di.swpout = kstat.pswpout << (PAGE_SHIFT - 10);
	
        di.loads[0] = LOAD_CONV (avenrun[0]);
        di.loads[1] = LOAD_CONV (avenrun[1]);
	di.loads[2] = LOAD_CONV (avenrun[2]);

	di.procnum = nr_threads-1;
	
	si_meminfo(&si);
	di.freeram   = si.freeram << (PAGE_SHIFT - 10);
        di.sharedram = si.sharedram;
	di.bufferram = si.bufferram << (PAGE_SHIFT - 10);
        di.cachedram = atomic_read(&page_cache_size) << (PAGE_SHIFT - 10);

	si_swapinfo(&si);
	di.totalswap = si.totalswap << (PAGE_SHIFT - 10);
        di.freeswap  = si.freeswap << (PAGE_SHIFT - 10);

        memcpy (page, &di, HOSTINFO_LEN);

        len = HOSTINFO_LEN;
	*start = page + offset;
        len -= offset;
        if(len > count) len = count;
	if (len < count) *eof = 1;
        if (len < 0) len = 0;

        return len;
}

static struct proc_dir_entry *sysinfo_proc_entry;

int sysmon_init(void)
{
	sysinfo_proc_entry = create_proc_read_entry("cis_sysinfo", 0, NULL, sysinfo_read_proc, NULL);
	if (!sysinfo_proc_entry) {
		printk("procmon_init: cannot register sysinfo\n");
		return -EPERM;
	}

//        printk (KERN_INFO "System monitoring enabled.\n");

        return 0;
}

int init_module(void)
{
        return sysmon_init ();
}

void cleanup_module(void)
{
        remove_proc_entry("cis_sysinfo", 0);
//	printk (KERN_INFO "System monitoring disabled.\n");
}

                