/*
 * Cluster Information Service - a monitoring system for Linux clusters
 * Copyright (C) 2000 Institute of Informatics, Slovak Academy of Sciences.
 * Written by Jan Astalos (astalos.ui@savba.sk)
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as published
 * by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Socket monitor. Provides CIS server with the information about connections
 * between processes (sockets).
 */

#include <sys/time.h>
#include <unistd.h>

#include "cis.h"
#include "mon_common.h"

int main(int argc, char *argv[])
{
	struct timeval timeout, interval_tv;
	fd_set fdset;

	monitor_type = SOCKMON;
	
        prepare_init(argc, argv);
	
	sockmon.init();

	if (sockmon.fd == -1)
		exit(1);

	connect_server();
	
	sockmon.send_init();
	recv_interval();
	sockmon.check();
        full_msg = FALSE;

#ifndef DEBUG
	daemonize();
#endif
	
	FD_ZERO(&fdset);
	FD_SET(socketfd, &fdset);
	if (sockmon.fd > -1)
		FD_SET(sockmon.fd, &fdset);

	sectotval(interval_tv, interval);
	timeout = interval_tv;
	
	for (;;) {
		switch (select (FD_SETSIZE, &fdset, NULL, NULL, &timeout)) {
		case 2:
		case 1:
			if (sockmon.fd > -1 && FD_ISSET(sockmon.fd, &fdset)) {
				sockmon.getmsg();
				break;
			}
		case 0:
			if (recv_msg())
				sockmon.send_init();
			sockmon.check();
			if (!full_msg)
				send_msg();
			if (!full_mode)
				full_msg = FALSE;
			timeout = interval_tv;
			break;
		case -1:
			timeout = interval_tv;
			break;
		default:
			break;
		}
		if (server_ok)
			FD_SET(socketfd, &fdset);
		else
			FD_CLR(socketfd, &fdset);
		if (sockmon.fd > -1)
			FD_SET(sockmon.fd, &fdset);
	}
}
